@@ -35,6 +35,7 @@ impl AtomicFile {
         if end == 0 {
             return;
         }
+        assert!(end == self.upd.size());
         let mut pos = 16;
         while pos < end {
             let start = self.upd.read_u64(pos);
@@ -51,8 +52,6 @@ impl AtomicFile {
     }
 }
 
-const TRACE: bool = false; // For debugging.
-
 impl Storage for AtomicFile {
     fn commit(&self, size: u64) {
         let mut map = self.map.lock().unwrap();
@@ -64,7 +63,7 @@ impl Storage for AtomicFile {
         // First set the end position to zero.
         self.upd.write_u64(0, 0);
         self.upd.write_u64(8, size);
-        self.upd.commit(16);
+        self.upd.commit(16); // Not clear if this is necessary.
 
         // Write the update records.
         let mut pos: u64 = 16;
@@ -78,14 +77,14 @@ impl Storage for AtomicFile {
             self.upd.write(pos, &v.data[v.off..v.off + v.len]);
             pos += len;
         }
-        self.upd.commit(pos);
+        self.upd.commit(pos); // Not clear if this is necessary.
 
         // Set the end position.
         self.upd.write_u64(0, pos);
         self.upd.write_u64(8, size);
         self.upd.commit(pos);
 
-        // Hopeflly updates are now securely stored in upd file.
+        // Hopefully updates are now securely stored in upd file.
 
         for (k, v) in map.iter() {
             let start = k + 1 - v.len as u64;
@@ -101,10 +100,6 @@ impl Storage for AtomicFile {
     }
 
     fn read(&self, start: u64, data: &mut [u8]) {
-        if TRACE {
-            println!("Reading start={} len={}", start, data.len());
-        }
-
         let mut todo: usize = data.len();
         if todo == 0 {
             return;
@@ -119,13 +114,6 @@ impl Storage for AtomicFile {
                 let amount = min(todo, lim) as usize;
                 self.stg
                     .read(start + done as u64, &mut data[done..done + amount]);
-                if TRACE {
-                    println!(
-                        "Read from underlying file at {} amount={}",
-                        start + done as u64,
-                        amount
-                    )
-                };
                 done += amount;
                 todo -= amount;
             }
@@ -136,14 +124,6 @@ impl Storage for AtomicFile {
                 let amount = min(todo, v.len - skip);
                 data[done..done + amount]
                     .copy_from_slice(&v.data[v.off + skip..v.off + skip + amount]);
-                if TRACE {
-                    println!(
-                        "Read from map start = {} amount={} skip={}",
-                        start + done as u64,
-                        amount,
-                        skip
-                    )
-                };
                 done += amount;
                 todo -= amount;
             }
@@ -154,20 +134,10 @@ impl Storage for AtomicFile {
         if todo > 0 {
             self.stg
                 .read(start + done as u64, &mut data[done..done + todo]);
-            if TRACE {
-                println!(
-                    "Read from underlying file at {} amount={}",
-                    start + done as u64,
-                    todo
-                )
-            };
         }
     }
 
     fn write_data(&self, start: u64, data: Data, off: usize, len: usize) {
-        if TRACE {
-            println!("write_data start={} len={}", start, len);
-        }
         if len == 0 {
             return;
         }
@@ -184,26 +154,17 @@ impl Storage for AtomicFile {
 
             // (a) New write ends before existing write.
             if end <= estart {
-                if TRACE {
-                    println!("Case (a) end={} estart={}", end, estart);
-                }
                 break;
             }
             // (b) New write subsumes existing write entirely, remove existing write.
             else if start <= estart && end >= eend {
                 remove.push(eend - 1);
-                if TRACE {
-                    println!("Case (b), removing {}", estart);
-                }
             }
             // (c) New write starts before existing write, but doesn't subsume it. Trim existing write.
             else if start <= estart {
                 let trim = (end - estart) as usize;
                 v.len -= trim;
                 v.off += trim;
-                if TRACE {
-                    println!("Case (c), estart={} v.len={}", estart, v.len);
-                }
             }
             // (d) New write starts in middle of existing write, ends before end of existing write...
             // .. put start of existing write in add list, trim existing write.
@@ -214,13 +175,6 @@ impl Storage for AtomicFile {
                 let trim = (end - estart) as usize;
                 v.len -= trim;
                 v.off += trim;
-
-                if TRACE {
-                    println!(
-                        "Case (d), estart={} remain={} v.len={}",
-                        estart, remain, v.len
-                    );
-                }
             }
             // (e) New write starts in middle of existing write, ends after existing write...
             // ... put start of existing write in add list, remove existing write,
@@ -229,10 +183,6 @@ impl Storage for AtomicFile {
                 add.push((estart, v.data.clone(), v.off, remain));
 
                 remove.push(eend - 1);
-
-                if TRACE {
-                    println!("Case (e), estart={} remain={}", estart, remain);
-                }
             }
         }
         for k in remove {
@@ -254,16 +204,17 @@ impl Storage for AtomicFile {
 
 #[test]
 pub fn test() {
+    use crate::stg::MemFile;
     use rand::Rng;
     /* Idea of test is to check AtomicFile and MemFile behave the same */
 
     let mut rng = rand::thread_rng();
 
     for _ in 0..1000 {
-        let s0 = Box::new(stg::MemFile::new());
-        let s1 = Box::new(stg::MemFile::new());
+        let s0 = Box::new(MemFile::default());
+        let s1 = Box::new(MemFile::default());
         let s2 = AtomicFile::new(s0, s1);
-        let s3 = stg::MemFile::new();
+        let s3 = MemFile::default();
 
         for _ in 0..1000 {
             let off: usize = rng.gen::<usize>() % 100;
